
# findFile - look for a file in the path

proc findFile {name path} {
    if {[file readable $name]} {
	return $name
    } elseif {[string index $name 0] == "/"} {
	return ""
    } else {
	foreach p $path {
	    set file "$p/$name"
	    if {[file readable $file]} {
		return $file
	    }
	}
    }
    return ""
}


# mkBasic w
#
# Create a top-level window that displays a basic text widget.
# Reads a file and tags line numbers
#
# Arguments:
#    w -	Name to use for new top-level window.
#    p1, p2     Patterns specifying which lines to take
#    

proc mkBasic {file {w .basic} {p1 ""}  {p2 ""}} {
    global text_window_font
    catch {destroy $w}
    toplevel $w
    wm title $w "Text Display - $file"
    wm iconname $w "Text Display"
    button $w.ok -text Dismiss -command "wm withdraw $w"
    text $w.t -relief raised -bd 2 -yscrollcommand "$w.s set" -setgrid true -font $text_window_font
    scrollbar $w.s -relief flat -command "$w.t yview"
    pack $w.ok -side bottom
    pack $w.s -side right -fill y
    pack $w.t -expand yes -fill both
    set str [open "$file" r]
    if {$str == 0} {
	puts stderr "Error: Couldn't open '$file'"
	return
    }
    set line_no 0
    set line_count 0
    if {$p1 != ""} {
	# read to first line specified by p1
	if {[regexp {^[     ]*([0-9]+)[     ]*$} $p1 all num]} {
	    while {$line_no < $num && [gets $str line] >= 0} {
		incr line_no
	    }
	} else {
	    while {[gets $str line] >= 0} {
		incr line_no
		if {[regexp $p1 $line]} break
	    }
	}	
	$w.t insert end "$line\n"
	set line_count 1
    }
    set max_width 0
    if {$p2 != ""} {
	# read to last line specified by p2
	if {[regexp {^[     ]*([0-9]+)[     ]*$} $p2 all num]} {
	    while {$line_no <= $num && [gets $str line] >= 0} {
		incr line_no
		incr line_count
		if {[string length $line] > $max_width} {set max_width [string length $line] }
		$w.t insert end "$line\n"
	    }
	} else {
	    while {[gets $str line] >= 0} {
		incr line_no
		incr line_count
		if {[string length $line] > $max_width} {set max_width [string length $line] }
		$w.t insert end "$line\n"
		if {[regexp $p2 $line]} break
	    }
	}
    } else {
	while {[gets $str line] >= 0} {
	    incr line_no
	    incr line_count
	    if {[string length $line] > $max_width} {set max_width [string length $line] }
	    $w.t insert end "$line\n"
	}
    }
    $w.t tag add hil 0.0 end
    if {$line_count < 40} { $w.t configure -height $line_count }
    set max_width [expr 5+$max_width]
    if {$max_width < 80} { $w.t configure -width $max_width }
    close $str
    bind $w <Any> ""
}

set text_win_id 0

# display text lines (open file if necessary)
# if to is specified, hilight region
# only create a window once for a file


proc disp_text_lines {file from to {pat ""}} {
    global file_windows
    if {! [info exists file_windows($file)]} {
	global text_win_id
	set win ".textwin$text_win_id"
	incr text_win_id
	set file_windows($file) "$win"
	mkBasic "$file" "$win"
    } else {
	set win $file_windows($file)
	wm deiconify $win
    }
    $win.t tag remove hil 0.0 end
    if {$pat != ""} {
	set n 1
	set last_line [$win.t index end]
	set from 0
	set to 0
	while {$n < $last_line} {
	    set line [$win.t get "$n.0" "$n.0 lineend"]
	    if {[regexp "$pat" $line]} {
		set from $n
		set to $n
		break
	    }
	    incr n
	}
    }
    $win.t yview [expr $from - 3]
    if {$to != ""} {
	incr to
	$win.t tag add hil "$from.0" "$to.0"
	$win.t tag configure hil -backgroun LightBlue
    }
}

proc delete_text_displays {} {
    global text_buffers file_windows
    if {[info exists text_buffers]} {unset text_buffers}
    if {[info exists file_windows]} {unset file_windows}
}

# display a subset of a file in a text window
# raise existing one if its there
# from and to are individually either numeric or regexps

proc disp_text_buffer {file from to} {
    global text_buffers
    if {[info exists text_buffers($file.$from.$to)]} {
	set win $text_buffers($file.$from.$to)
	set stat [wm state $win]
	if {$stat == "withdrawn" || $stat == "iconic"} {
	    wm deiconify $win
	}
	return
    } else {
	global text_win_id
	set win ".textwin$text_win_id"
	incr text_win_id
	set text_buffers($file.$from.$to) "$win"
	mkBasic "$file" "$win" $from $to
    }
}


# if the spec is a text spec, display the file and return 1
# else return 0

proc is_text_spec {spec {path ""} {notfile 1}} {
    if {! $notfile} {
	return [do_text_specs $spec $path 0]
    } elseif {[regexp {^text[;:]([^:;,]+)[,:;]([0-9]+)(,([0-9]+))?} $spec all filename from misc to]} {
	set file [findFile $filename $path]
	if {$file == ""} {
	    puts stderr "File '$filename' not found"
	    return 0
	}
	if {$to == ""} {set to $from}
	disp_text_lines "$file" $from $to
	return 1
    } elseif {[regexp {^text:([^;]+);(.*)$} $spec all filename pat]} {
	set file [findFile $filename $path]
	if {$file == ""} {
	    puts stderr "File '$filename' not found"
	    return 0
	}
	disp_text_lines "$file" {} {} "$pat" 

    } elseif {[regexp {^image:([^;: 	]+)} $spec all filename]} {
	set file [findFile $filename $path]
	if {$file == ""} {
	    puts stderr "File '$filename' not found"
	    return 0
	}
	exec xv [glob $file] &
	return 1
    } elseif {[regexp {^html:([^;: 	]+)} $spec all filename]} {
	set file [findFile $filename $path]
	if {$file == ""} {
	    puts stderr "File '$filename' not found"
	    return 0
	}
	display_html $file
	return 1
    } else {
	return 0
    }
}

set dts_pat1 {^.*"([^   :;]+)[  :;]+([^,]+)([,  ]+([^,"]+))?}
set dts_pat2 {^[     ]*([^   :;]+)[  :;]+([^,]+)([,  ]+([^,]+))?}

proc do_text_specs {specs path {notfile 1}} {
    global dts_pat1 dts_pat2
    set result 0
    foreach spec $specs {
	if {[regexp "^\{" $spec]} {
	    do_text_specs $spec $path $notfile
	} elseif {! $notfile} {
	    if {[is_text_spec "text:$spec" $path 1]} {
		set result 1
	    } elseif {[is_text_spec "image:$spec" $path 1]} {
		set result 1
	    } elseif {[is_text_spec "$spec" $path 1]} {
		set result 1
	    } else {
		puts stdout "$spec - not a file spec"
	    }
	} elseif {([regexp $dts_pat1 $spec all filename from misc to] ||
		   [regexp $dts_pat2 $spec all filename from misc to])} {
	    set file [findFile $filename $path]
	    if {$file == ""} {
		puts stderr "File '$filename' not found"
		continue
	    }
	    if {$to == ""} {set to $from}
	    disp_text_buffer "$file" $from $to
	    set result 1
	} else {
	    puts stdout "$spec - not a file spec"
	}
    }
    return $result
}

#Local Variables:
#mode: tcl
#End:


